/* eslint-disable no-magic-numbers */
import Image from 'next/image';
import React, { useEffect, useState } from 'react';
import lensIcon from '@/assets/vectors/icons/lens.svg';
import { twMerge } from 'tailwind-merge';

export type CheckboxItem = { id: string; label: string };

type CheckboxFilterProps = {
  options: CheckboxItem[];
  onFilterChange?: (filteredItems: CheckboxItem[]) => void;
  onCheckedChange?: (filteredItems: CheckboxItem[]) => void;
  isSearchEnabled?: boolean;
};

export const CheckboxFilter = ({
  options,
  onFilterChange,
  onCheckedChange,
  isSearchEnabled = true,
}: CheckboxFilterProps): React.ReactNode => {
  const [searchTerm, setSearchTerm] = useState('');
  const [filteredOptions, setFilteredOptions] = useState<CheckboxItem[]>(options);
  const [checkedCheckboxes, setCheckedCheckboxes] = useState<CheckboxItem[]>([]);

  const filterOptions = (term: string): void => {
    const filteredItems = options.filter(item =>
      item.label.toLowerCase().includes(term.toLowerCase()),
    );

    setFilteredOptions(filteredItems);
    onFilterChange?.(filteredItems);
  };

  const handleSearchTermChange = (e: React.ChangeEvent<HTMLInputElement>): void => {
    const newSearchTerm = e.target.value;
    setSearchTerm(newSearchTerm);
    filterOptions(newSearchTerm);
  };

  const handleCheckboxChange = (option: CheckboxItem): void => {
    const newCheckedCheckboxes = checkedCheckboxes.includes(option)
      ? checkedCheckboxes.filter(item => item !== option)
      : [...checkedCheckboxes, option];

    setCheckedCheckboxes(newCheckedCheckboxes);
    onCheckedChange?.(newCheckedCheckboxes);
  };

  useEffect(() => {
    setFilteredOptions(options);
  }, [options]);

  return (
    <div className="relative" data-testid="checkbox-filter">
      {isSearchEnabled && (
        <div className="relative" data-testid="search">
          <input
            name="search-input"
            aria-label="search-input"
            value={searchTerm}
            onChange={handleSearchTermChange}
            placeholder="Search..."
            className="h-9 w-full rounded-[64px] border border-transparent bg-cultured px-4 py-2.5 text-xs font-medium text-font-400 outline-none  hover:border-greyscale-600 focus:border-greyscale-600"
          />

          <Image
            src={lensIcon}
            alt="lens icon"
            height={16}
            width={16}
            className="absolute right-4 top-2.5"
          />
        </div>
      )}

      <div
        className={twMerge(
          'mb-6 max-h-[300px] overflow-y-auto py-2.5 pr-2.5',
          isSearchEnabled && 'mt-6',
        )}
      >
        {filteredOptions.length === 0 ? (
          <p className="w-full text-sm text-font-400">No matching elements found.</p>
        ) : (
          <ul className="columns-2 gap-8">
            {filteredOptions.map(option => (
              <li key={option.id} className="mb-5 flex items-center gap-x-2">
                <input
                  type="checkbox"
                  id={option.id}
                  className=" h-4 w-4 shrink-0 accent-primary-500"
                  onChange={(): void => handleCheckboxChange(option)}
                />
                <label htmlFor={option.id} className="break-all text-sm">
                  {option.label}
                </label>
              </li>
            ))}
          </ul>
        )}
      </div>
    </div>
  );
};
