/* eslint-disable no-magic-numbers */
import {
  createdOverlayFileSchema,
  createdOverlaySchema,
  mapOverlay,
  uploadedOverlayFileContentSchema,
} from '@/models/mapOverlay';
import { axiosInstance } from '@/services/api/utils/axiosInstance';
import { CreatedOverlay, CreatedOverlayFile, MapOverlay } from '@/types/models';
import { validateDataUsingZodSchema } from '@/utils/validateDataUsingZodSchema';
import { createAsyncThunk } from '@reduxjs/toolkit';
import { z } from 'zod';
import { apiPath } from '../apiPath';
import { CHUNK_SIZE } from './overlays.constants';

export const getAllPublicOverlaysByProjectId = createAsyncThunk(
  'overlays/getAllPublicOverlaysByProjectId',
  async (projectId: string): Promise<MapOverlay[]> => {
    const response = await axiosInstance.get<MapOverlay[]>(
      apiPath.getAllOverlaysByProjectIdQuery(projectId, { publicOverlay: true }),
    );

    const isDataValid = validateDataUsingZodSchema(response.data, z.array(mapOverlay));

    return isDataValid ? response.data : [];
  },
);

/** UTILS */

type CreateFileArgs = {
  filename: string;
  content: string;
};

const createFile = async ({ filename, content }: CreateFileArgs): Promise<CreatedOverlayFile> => {
  const fileParams = {
    filename: `C:\\fakepath\\${filename}`,
    length: content.length.toString(),
  };

  const response = await axiosInstance.post(
    apiPath.createOverlayFile(),
    new URLSearchParams(fileParams),
    {
      withCredentials: true,
    },
  );

  const isDataValid = validateDataUsingZodSchema(response.data, createdOverlayFileSchema);
  return isDataValid ? response.data : undefined;
};

type UploadContentArgs = {
  createdFile: CreatedOverlayFile;
  overlayContent: string;
};
const uploadContent = async ({ createdFile, overlayContent }: UploadContentArgs): Promise<void> => {
  const data = new Uint8Array(new TextEncoder().encode(overlayContent));
  let uploadedLength = 0;

  const sendChunk = async (): Promise<void> => {
    if (uploadedLength >= data.length) {
      return;
    }

    const chunk = data.slice(uploadedLength, uploadedLength + CHUNK_SIZE);

    const responeJSON = await fetch(
      `${process.env.NEXT_PUBLIC_BASE_API_URL}/${apiPath.uploadOverlayFileContent(createdFile.id)}`,
      {
        method: 'POST',
        credentials: 'include',
        body: chunk,
      },
    );

    const response = await responeJSON.json();
    validateDataUsingZodSchema(response, uploadedOverlayFileContentSchema);

    uploadedLength += chunk.length;
    sendChunk();
  };

  await sendChunk();
};

type CreatedOverlayArgs = {
  createdFile: CreatedOverlayFile;
  description: string;
  name: string;
  type: string;
  projectId: string;
};

const creteOverlay = async ({
  createdFile,
  description,
  type,
  name,
  projectId,
}: CreatedOverlayArgs): Promise<CreatedOverlay> => {
  const data = {
    name,
    description,
    filename: createdFile.filename,
    googleLicenseConsent: false.toString(),
    type,
    fileId: createdFile.id.toString(),
  };

  const overlay = new URLSearchParams(data);

  const response = await axiosInstance.post(apiPath.createOverlay(projectId), overlay, {
    withCredentials: true,
  });

  const isDataValid = validateDataUsingZodSchema(response.data, createdOverlaySchema);

  return isDataValid ? response.data : undefined;
};

type AddOverlayArgs = {
  filename: string;
  content: string;
  description: string;
  type: string;
  name: string;
  projectId: string;
};

export const addOverlay = createAsyncThunk(
  'overlays/addOverlay',
  async ({
    filename,
    content,
    description,
    name,
    type,
    projectId,
  }: AddOverlayArgs): Promise<void> => {
    const createdFile = await createFile({
      filename,
      content,
    });

    await uploadContent({
      createdFile,
      overlayContent: content,
    });

    await creteOverlay({
      createdFile,
      description,
      name,
      type,
      projectId,
    });
  },
);
